/***********************************************************
Bump Gloss.fx - Bump Gloss Shader
By: Preston "NWC_Omega" Chaderton

This bump gloss shader is highly based on the
FX Composer bump map tutorial.  To a point 
where I don't really know how it works.  It 
only has one pass.

It was created in FX Composer and then in Notepad for use in
Dark Basic Pro.  It compiles under VS 1.1 and PS 1.4 for the
less to mid powerful computers, and it runs quite well.

It runs at 280 FPS on my Intel Celeron 1.3 Ghrz, 512 MB
Ram, NVIDIA GeForce FX 5200 when applied to a sphere 
with 3278 triangles in a test program running at 
640x480x32.

***********************************************************/


// light direction (view space)
float3 lightDir  = {0.0f, 0.0f, 1.0f};

// light intensity
float4 I_a = { 0.1f, 0.1f, 0.1f, 1.0f };    // ambient
float4 I_d = { 0.5f, 0.5f, 0.5f, 0.5f };    // diffuse
float4 I_s = { 1.0f, 1.0f, 1.0f, 1.0f };    // specular

// material reflectivity
float4 k_a = { 1.0f, 1.0f, 1.0f, 1.0f};
float4 k_d = {1.0f, 1.0f, 1.0f, 1.0f};
float4 k_s = { 1.0f, 1.0f, 1.0f, 1.0f};
int n = 64;

float bumpScale = 1.0f;

// texture
texture Main < 
                string texturetype = "2D" ; 
                string name = "base.dds"; 
             >;

texture Norm <
				string texturetype = "2D" ;
				string name = "bump.dds";
			 >;

texture Glos <
				string texturetype = "2D" ;
				string name = "gloss.dds";
			 >;
			 
texture Phon <
				string texturetype = "2D" ;
				string name = "phong.dds";
			  >;

// transformations
float4x4 World      : WORLD;
float4x4 View       : VIEW;
float4x4 Projection : PROJECTION;
float4x4 WorldI     : WORLDINVERSE;
float4x4 ViewI      : VIEWINVERSE;

struct VS_OUTPUT
{
    float4 Pos  : POSITION;
    float4 Diff : COLOR0;
    float4 Spec : COLOR1;
    float2 Tex  : TEXCOORD0;
    float2 TexB : TEXCOORD1;
    float2 TexC : TEXCOORD2;
    float3 LB   : TEXCOORD3; 
	float3 HB   : TEXCOORD4;
};

VS_OUTPUT VS(
    float3 Pos  : POSITION, 
    float3 Norm : NORMAL, 
    float2 Tex  : TEXCOORD0,
    float3 Tan  : TANGENT,
    float3 Bin  : BINORMAL)
{
    VS_OUTPUT Out = (VS_OUTPUT)0;

    float3 L = -lightDir;

    float4x4 WorldView = mul(World, View);

    float3 P = mul(float4(Pos, 1), (float4x3)WorldView);  // position (view space)
    float3 N = normalize(mul(Norm, (float3x3)World)); // normal (view space)

    float3 R = normalize(2 * dot(N, L) * N - L);          // reflection vector (view space)
    float3 V = -normalize(P);                             // view direction (view space)

	// compute the 3x3 tranform from object space to tangent space
	float3x3 objToTangentSpace;
	objToTangentSpace[0] = Tan * bumpScale;
	objToTangentSpace[1] = Bin * bumpScale;
	objToTangentSpace[2] = Norm;

    float4 vertexPos = mul(Pos, World); // world space position

	// light vector
    float3 lightVec = -lightDir;
	lightVec = mul(lightVec, (float3x3) WorldI);  // transform back to object space
	Out.LB = mul(objToTangentSpace, lightVec); // transform from object to tangent space
	
	// eye vector
	float3 eyeVec = ViewI[3].xyz - vertexPos.xyz; // world space eye vector
	eyeVec = mul(eyeVec, (float3x3) WorldI);  // transform back to object space
	eyeVec = normalize(eyeVec);
	
	// half-angle vector
	float3 H = normalize(lightVec + eyeVec);
	Out.HB = mul(objToTangentSpace, H);	// transform to tangent space


    Out.Pos  = mul(float4(P, 1), Projection);             // position (projected)
    Out.Diff = I_a * k_a + I_d * k_d * max(0, dot(N, L)); // diffuse + ambient
    Out.Spec = I_s * k_s * pow(max(0, dot(R, V)), n/4);   // specular
    Out.Tex  = Tex; 
    Out.TexB = Tex;
    Out.TexC = Tex;                                      

    return Out;
}

sampler MainSamp = sampler_state
{
    Texture   = (Main);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
};

sampler NormSamp = sampler_state
{
    Texture   = (Norm);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
};

sampler GlosSamp = sampler_state
{
    Texture   = (Glos);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
};

sampler PhonSamp = sampler_state
{
    Texture   = (Phon);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU  = CLAMP;        
    AddressV  = CLAMP;
    AddressW  = CLAMP;
};

float4 PS(
    float4 Diff : COLOR0,
    float4 Spec : COLOR1,
    float2 Tex  : TEXCOORD0,
    float2 TexB : TEXCOORD1,
    float2 TexC : TEXCOORD2,
    float3 L    : TEXCOORD3,
    float3 H    : TEXCOORD4) : COLOR
{
    half4 colorMap = tex2D(MainSamp, Tex);
	half3 N = tex2D(NormSamp, TexB)*2.0 - 1.0;
	
	half NdotL = dot(N, L);
	half NdotH = dot(N, H);
	half4 light = tex2D(PhonSamp, half2(NdotL, NdotH));
	
	//return I_a*colorMap + light*Diff*colorMap + light.a*Spec;
    
    return I_a * colorMap + light*Diff*colorMap + Diff*(Spec*tex2D(GlosSamp,TexC));
    
    //return tex2D(MainSamp, Tex)*Diff+(Spec*tex2D(GlosSamp,TexC));
}

technique TVertexAndPixelShader
{
    pass P0
    {
        // shaders
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_1_4 PS();
    }  
}
